@@ -25,6 +25,7 @@
 #include "../lout/debug.hh"
 #include "../lout/misc.hh"
 
+using namespace lout;
 using namespace lout::container;
 using namespace lout::object;
 
@@ -65,11 +66,115 @@ void Layout::Emitter::emitCanvasSizeChanged (int width,
    emitVoid (CANVAS_SIZE_CHANGED, 3, argv);
 }
 
+// ----------------------------------------------------------------------
+
+bool Layout::LinkReceiver::enter (Widget *widget, int link, int img,
+                                  int x, int y)
+{
+   return false;
+}
+
+bool Layout::LinkReceiver::press (Widget *widget, int link, int img,
+                                  int x, int y, EventButton *event)
+{
+   return false;
+}
+
+bool Layout::LinkReceiver::release (Widget *widget, int link, int img,
+                                    int x, int y, EventButton *event)
+{
+   return false;
+}
+
+bool Layout::LinkReceiver::click (Widget *widget, int link, int img,
+                                    int x, int y, EventButton *event)
+{
+   return false;
+}
+
+// ----------------------------------------------------------------------
+
+bool Layout::LinkEmitter::emitToReceiver (lout::signal::Receiver *receiver,
+                                          int signalNo, int argc,
+                                          lout::object::Object **argv)
+{
+   LinkReceiver *linkReceiver = (LinkReceiver*)receiver;
+
+   switch (signalNo) {
+   case ENTER:
+      return linkReceiver->enter ((Widget*)argv[0],
+                                  ((Integer*)argv[1])->getValue (),
+                                  ((Integer*)argv[2])->getValue (),
+                                  ((Integer*)argv[3])->getValue (),
+                                  ((Integer*)argv[4])->getValue ());
+
+   case PRESS:
+      return linkReceiver->press ((Widget*)argv[0],
+                                  ((Integer*)argv[1])->getValue (),
+                                  ((Integer*)argv[2])->getValue (),
+                                  ((Integer*)argv[3])->getValue (),
+                                  ((Integer*)argv[4])->getValue (),
+                                  (EventButton*)argv[5]);
+
+   case RELEASE:
+      return linkReceiver->release ((Widget*)argv[0],
+                                    ((Integer*)argv[1])->getValue (),
+                                    ((Integer*)argv[2])->getValue (),
+                                    ((Integer*)argv[3])->getValue (),
+                                    ((Integer*)argv[4])->getValue (),
+                                    (EventButton*)argv[5]);
+
+   case CLICK:
+      return linkReceiver->click ((Widget*)argv[0],
+                                  ((Integer*)argv[1])->getValue (),
+                                  ((Integer*)argv[2])->getValue (),
+                                  ((Integer*)argv[3])->getValue (),
+                                  ((Integer*)argv[4])->getValue (),
+                                  (EventButton*)argv[5]);
+
+   default:
+      misc::assertNotReached ();
+   }
+   return false;
+}
+
+bool Layout::LinkEmitter::emitEnter (Widget *widget, int link, int img,
+                                     int x, int y)
+{
+   Integer ilink (link), iimg (img), ix (x), iy (y);
+   Object *argv[5] = { widget, &ilink, &iimg, &ix, &iy };
+   return emitBool (ENTER, 5, argv);
+}
+
+bool Layout::LinkEmitter::emitPress (Widget *widget, int link, int img,
+                                     int x, int y, EventButton *event)
+{
+   Integer ilink (link), iimg (img), ix (x), iy (y);
+   Object *argv[6] = { widget, &ilink, &iimg, &ix, &iy, event };
+   return emitBool (PRESS, 6, argv);
+}
+
+bool Layout::LinkEmitter::emitRelease (Widget *widget, int link, int img,
+                                       int x, int y, EventButton *event)
+{
+   Integer ilink (link), iimg (img), ix (x), iy (y);
+   Object *argv[6] = { widget, &ilink, &iimg, &ix, &iy, event };
+   return emitBool (RELEASE, 6, argv);
+}
+
+bool Layout::LinkEmitter::emitClick (Widget *widget, int link, int img,
+                                     int x, int y, EventButton *event)
+{
+   Integer ilink (link), iimg (img), ix (x), iy (y);
+   Object *argv[6] = { widget, &ilink, &iimg, &ix, &iy, event };
+   return emitBool (CLICK, 6, argv);
+}
+
 // ---------------------------------------------------------------------
 
 Layout::Anchor::~Anchor ()
 {
-   delete name;
+   free(name);
 }
 
 // ---------------------------------------------------------------------
@@ -77,7 +182,7 @@ Layout::Anchor::~Anchor ()
 Layout::Layout (Platform *platform)
 {
    this->platform = platform;
-   views = new container::typed::List <View> (true);
+   view = NULL;
    topLevel = NULL;
    widgetAtPoint = NULL;
 
@@ -89,6 +194,7 @@ Layout::Layout (Platform *platform)
    canvasWidth = canvasAscent = canvasDescent = 0;
 
    usesViewport = false;
+   drawAfterScrollReq = false;
    scrollX = scrollY = 0;
    viewportWidth = viewportHeight = 0;
    hScrollbarThickness = vScrollbarThickness = 0;
@@ -114,15 +220,21 @@ Layout::Layout (Platform *platform)
 
 Layout::~Layout ()
 {
+   widgetAtPoint = NULL;
+
    if (scrollIdleId != -1)
       platform->removeIdle (scrollIdleId);
    if (resizeIdleId != -1)
       platform->removeIdle (resizeIdleId);
-
-   if (topLevel)
-      delete topLevel;
+   if (bgColor)
+      bgColor->unref ();
+   if (topLevel) {
+      Widget *w = topLevel;
+      topLevel = NULL;
+      delete w;
+   }
    delete platform;
-   delete views;
+   delete view;
    delete anchorsTable;
    delete textZone;
 }
@@ -141,7 +253,6 @@ void Layout::addWidget (Widget *widget)
 
    canvasHeightGreater = false;
    setSizeHints ();
-   updateBgColor ();
    queueResize ();
 }
 
@@ -155,13 +266,10 @@ void Layout::removeWidget ()
    canvasWidth = canvasAscent = canvasDescent = 0;
    scrollX = scrollY = 0;
 
-   for (typed::Iterator <View> it = views->iterator (); it.hasNext (); ) {
-      View *view = it.getNext ();
-      view->setCanvasSize (canvasWidth, canvasAscent, canvasDescent);
-      if (view->usesViewport ())
-         view->setViewportSize (viewportWidth, viewportHeight, 0, 0);
-      view->queueDrawTotal ();
-   }
+   view->setCanvasSize (canvasWidth, canvasAscent, canvasDescent);
+   if (view->usesViewport ())
+      view->setViewportSize (viewportWidth, viewportHeight, 0, 0);
+   view->queueDrawTotal ();
 
    setAnchor (NULL);
    updateAnchor ();
@@ -176,9 +284,12 @@ void Layout::removeWidget ()
 
 void Layout::setWidget (Widget *widget)
 {
-   if (topLevel)
-      delete topLevel;
    widgetAtPoint = NULL;
+   if (topLevel) {
+      Widget *w = topLevel;
+      topLevel = NULL;
+      delete w;
+   }
    textZone->zoneFree ();
    addWidget (widget);
 
@@ -193,7 +304,10 @@ void Layout::setWidget (Widget *widget)
  */
 void Layout::attachView (View *view)
 {
-   views->append (view);
+   if (this->view)
+      MSG_ERR("attachView: Multiple views for layout!\n");
+
+   this->view = view;
    platform->attachView (view);
 
    /*
@@ -235,11 +349,12 @@ void Layout::attachView (View *view)
 
 void Layout::detachView (View *view)
 {
+   if (this->view != view)
+      MSG_ERR("detachView: this->view: %p view %p\n", this->view, view);
+
    view->setLayout (NULL);
    platform->detachView (view);
-
-   views->detachRef (view);
-
+   this->view = NULL;
    /**
     * \todo Actually, viewportMarkerWidthDiff and
     *       viewportMarkerHeightDiff have to be recalculated here, since the
@@ -248,6 +363,12 @@ void Layout::detachView (View *view)
     */
 }
 
+void Layout::scroll(ScrollCommand cmd)
+{
+   if (view->usesViewport ())
+      view->scroll(cmd);
+}
+
 /**
  * \brief Scrolls all viewports, so that the region [x, y, width, height]
  *    is seen, according to hpos and vpos.
@@ -335,10 +456,10 @@ void Layout::scrollIdle ()
 
    if (xChanged || yChanged) {
       adjustScrollPos ();
-      for (container::typed::Iterator <View> it = views->iterator ();
-           it.hasNext (); ) {
-         View *thisView = it.getNext();
-         thisView->scrollTo (scrollX, scrollY);
+      view->scrollTo (scrollX, scrollY);
+      if (drawAfterScrollReq) {
+         drawAfterScrollReq = false;
+         view->queueDrawTotal ();
       }
    }
 
@@ -387,7 +508,11 @@ void Layout::draw (View *view, Rectangle *area)
 {
    Rectangle widgetArea, intersection, widgetDrawArea;
 
-   if (topLevel) {
+   if (scrollIdleId != -1) {
+      /* scroll is pending, defer draw until after scrollIdle() */
+      drawAfterScrollReq = true;
+
+   } else if (topLevel) {
       /* Draw the top level widget. */
       widgetArea.x = topLevel->allocation.x;
       widgetArea.y = topLevel->allocation.y;
@@ -418,7 +543,7 @@ void Layout::setAnchor (const char *anchor)
    _MSG("setAnchor (%s)\n", anchor);
 
    if (requestedAnchor)
-      delete requestedAnchor;
+      free(requestedAnchor);
    requestedAnchor = anchor ? strdup (anchor) : NULL;
    updateAnchor ();
 }
@@ -489,11 +614,7 @@ void Layout::setCursor (style::Cursor cursor)
 {
    if (cursor != this->cursor) {
       this->cursor = cursor;
-
-      for (typed::Iterator <View> it = views->iterator (); it.hasNext (); ) {
-         View *view = it.getNext ();
-         view->setCursor (cursor);
-      }
+      view->setCursor (cursor);
    }
 }
 
@@ -505,21 +626,17 @@ void Layout::updateCursor ()
       setCursor (style::CURSOR_DEFAULT);
 }
 
-void Layout::updateBgColor ()
+void Layout::setBgColor (style::Color *color)
 {
-   /* The toplevel widget should always have a defined background color,
-    * except at the beginning. Searching a defined background is not
-    * necessary. */
-   if (topLevel && topLevel->getStyle() &&
-       topLevel->getStyle()->backgroundColor)
-      bgColor = topLevel->getStyle()->backgroundColor;
-   else
-      bgColor = NULL;
+   color->ref ();
 
-   for (typed::Iterator <View> it = views->iterator (); it.hasNext (); ) {
-      View *view = it.getNext ();
+   if (bgColor)
+      bgColor->unref ();
+
+   bgColor = color;
+
+   if (view)
       view->setBgColor (bgColor);
-   }
 }
 
 void Layout::resizeIdle ()
@@ -551,12 +668,9 @@ void Layout::resizeIdle ()
          emitter.emitCanvasSizeChanged (
             canvasWidth, canvasAscent, canvasDescent);
 
-         // Tell the views about the new world size.
-         for (typed::Iterator <View> it = views->iterator (); it.hasNext ();) {
-            View *view = it.getNext ();
-            view->setCanvasSize (canvasWidth, canvasAscent, canvasDescent);
-          //  view->queueDrawTotal (false);
-         }
+         // Tell the view about the new world size.
+         view->setCanvasSize (canvasWidth, canvasAscent, canvasDescent);
+         //  view->queueDrawTotal (false);
 
          if (usesViewport) {
             int actualHScrollbarThickness =
@@ -574,14 +688,9 @@ void Layout::resizeIdle ()
             }
 
             // Set viewport sizes.
-            for (typed::Iterator <View> it = views->iterator ();
-               it.hasNext (); ) {
-               View *view = it.getNext ();
-               if (view->usesViewport ())
-                  view->setViewportSize (viewportWidth, viewportHeight,
-                                         actualHScrollbarThickness,
-                                         actualVScrollbarThickness);
-            }
+            view->setViewportSize (viewportWidth, viewportHeight,
+                                   actualHScrollbarThickness,
+                                   actualVScrollbarThickness);
          }
       }
 
@@ -597,7 +706,7 @@ void Layout::setSizeHints ()
    if (topLevel) {
       topLevel->setWidth (viewportWidth
                           - (canvasHeightGreater ? vScrollbarThickness : 0));
-      topLevel->setAscent (viewportHeight - vScrollbarThickness);
+      topLevel->setAscent (viewportHeight - hScrollbarThickness);
       topLevel->setDescent (0);
    }
 }
@@ -612,11 +721,7 @@ void Layout::queueDraw (int x, int y, int width, int height)
 
    if (area.isEmpty ()) return;
 
-   for (container::typed::Iterator <View> it = views->iterator ();
-      it.hasNext (); ) {
-      View *view = it.getNext ();
-      view->queueDraw (&area);
-   }
+   view->queueDraw (&area);
 }
 
 void Layout::queueDrawExcept (int x, int y, int width, int height,
@@ -645,11 +750,7 @@ void Layout::queueDrawExcept (int x, int y, int width, int height,
 void Layout::queueResize ()
 {
    if (resizeIdleId == -1) {
-      for (container::typed::Iterator <View> it = views->iterator ();
-           it.hasNext (); ) {
-         View *view = it.getNext ();
-         view->cancelQueueDraw ();
-      }
+      view->cancelQueueDraw ();
 
       resizeIdleId = platform->addIdle (&Layout::resizeIdle);
    }
@@ -722,6 +823,7 @@ void Layout::enterNotify (View *view, int x, int y, ButtonState state)
  */
 void Layout::leaveNotify (View *view, ButtonState state)
 {
+#if 0
    Widget *lastWidget;
    EventCrossing event;
 
@@ -734,6 +836,9 @@ void Layout::leaveNotify (View *view, ButtonState state)
       event.currentWidget = widgetAtPoint;
       lastWidget->leaveNotify (&event);
    }
+#else
+   moveOutOfView (state);
+#endif
 }
 
 /*
@@ -752,15 +857,16 @@ Widget *Layout::getWidgetAtPoint (int x, int y)
 
 /*
  * Emit the necessary crossing events, when the mouse pointer has moved to
- * the given widget.
+ * the given widget (by mouse or scrolling).
  */
 void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
 {
    Widget *ancestor, *w;
    Widget **track;
-   int trackLen, i;
+   int trackLen, i, i_a;
    EventCrossing crossingEvent;
 
+   _MSG("moveToWidget: wap=%p nwap=%p\n",widgetAtPoint,newWidgetAtPoint);
    if (newWidgetAtPoint != widgetAtPoint) {
       // The mouse pointer has been moved into another widget.
       if (newWidgetAtPoint && widgetAtPoint)
@@ -789,6 +895,7 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
          /* first part */
          for (w = widgetAtPoint; w != ancestor; w = w->getParent ())
             track[i++] = w;
+      i_a = i;
       track[i++] = ancestor;
       if (newWidgetAtPoint) {
          /* second part */
@@ -796,17 +903,32 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
          for (w = newWidgetAtPoint; w != ancestor; w = w->getParent ())
             track[i--] = w;
       }
+#if 0
+      MSG("Track: %s[ ", widgetAtPoint ? "" : "nil ");
+      for (i = 0; i < trackLen; i++)
+         MSG("%s%p ", i == i_a ? ">" : "", track[i]);
+      MSG("] %s\n", newWidgetAtPoint ? "" : "nil");
+#endif
 
-      /* Send events to all events on the track */
+      /* Send events to the widgets on the track */
       for (i = 0; i < trackLen; i++) {
          crossingEvent.state = state;
          crossingEvent.currentWidget = widgetAtPoint; // ???
          crossingEvent.lastWidget = widgetAtPoint; // ???
-
-         if (i != 0)
-            track[i]->enterNotify (&crossingEvent);
-         if (i != trackLen - 1)
+         if (i < i_a) {
             track[i]->leaveNotify (&crossingEvent);
+         } else if (i == i_a) { /* ancestor */
+            /* Don't touch ancestor unless:
+             *   - moving into/from NULL,
+             *   - ancestor becomes the newWidgetAtPoint */
+            if (i_a == trackLen-1 && !newWidgetAtPoint)
+               track[i]->leaveNotify (&crossingEvent);
+            else if ((i_a == 0 && !widgetAtPoint) ||
+                     (i_a == trackLen-1 && newWidgetAtPoint))
+               track[i]->enterNotify (&crossingEvent);
+         } else {
+            track[i]->enterNotify (&crossingEvent);
+         }
       }
 
       delete[] track;
@@ -847,6 +969,10 @@ bool Layout::processMouseEvent (MousePositionEvent *event,
          }
       }
    }
+   if (type == BUTTON_PRESS)
+      return emitLinkPress (NULL, -1, -1, -1, -1, (EventButton*)event);
+   else if (type == BUTTON_RELEASE)
+      return emitLinkRelease(NULL, -1, -1, -1, -1, (EventButton*)event);
 
    return false;
 }
@@ -862,14 +988,6 @@ void Layout::scrollPosChanged (View *view, int x, int y)
       scrollX = x;
       scrollY = y;
 
-      // Tell all views about the scrolling position, except the caller.
-      for (container::typed::Iterator <View> it = views->iterator ();
-           it.hasNext (); ) {
-         View *thisView = it.getNext();
-         if (view != thisView && thisView->usesViewport ())
-            thisView->scrollTo (scrollX, scrollY);
-      }
-
       setAnchor (NULL);
       updateAnchor ();
    }
@@ -898,21 +1016,6 @@ void Layout::viewportSizeChanged (View *view, int width, int height)
    viewportHeight = height;
 
    setSizeHints ();
-
-   int actualHScrollbarThickness =
-      (canvasWidth > viewportWidth) ? hScrollbarThickness : 0;
-   int actualVScrollbarThickness =
-      (canvasAscent + canvasDescent > viewportWidth) ? vScrollbarThickness : 0;
-
-   /* Tell all views about the size, except the caller. */
-   for (container::typed::Iterator <View> it = views->iterator ();
-        it.hasNext (); ) {
-      View *thisView = it.getNext();
-      if (view != thisView && thisView->usesViewport ())
-         thisView->setViewportSize (viewportWidth, viewportHeight,
-                                    actualHScrollbarThickness,
-                                    actualVScrollbarThickness);
-   }
 }
 
 } // namespace dw
