@@ -10,10 +10,9 @@
  * (at your option) any later version.
  */
 
-/* Handling of cookies takes place here.
- * This implementation aims to follow RFC 2965:
- * http://www.ietf.org/rfc/rfc2965.txt
- */
+/* Handling of cookies takes place here. */
+
+#include "msg.h"
 
 #ifdef DISABLE_COOKIES
 
@@ -37,12 +36,10 @@ void a_Cookies_init(void)
 #include <ctype.h>
 #include <errno.h>
 
-#include "msg.h"
 #include "IO/Url.h"
 #include "list.h"
 #include "cookies.h"
 #include "capi.h"
-#include "dpiapi.h"
 #include "../dpip/dpip.h"
 
 
@@ -139,10 +136,11 @@ void a_Cookies_freeall()
 /*
  * Set the value corresponding to the cookie string
  */
-void a_Cookies_set(Dlist *cookie_strings, const DilloUrl *set_url)
+void a_Cookies_set(Dlist *cookie_strings, const DilloUrl *set_url,
+                   const char *date)
 {
    CookieControlAction action;
-   char *cmd, *cookie_string, *dpip_tag, numstr[16];
+   char *cmd, *cookie_string, *dpip_tag;
    const char *path;
    int i;
 
@@ -157,10 +155,14 @@ void a_Cookies_set(Dlist *cookie_strings, const DilloUrl *set_url)
 
    for (i = 0; (cookie_string = dList_nth_data(cookie_strings, i)); ++i) {
       path = URL_PATH_(set_url);
-      snprintf(numstr, 16, "%d", URL_PORT(set_url));
-      cmd = a_Dpip_build_cmd("cmd=%s cookie=%s host=%s path=%s port=%s",
-                             "set_cookie", cookie_string, URL_HOST_(set_url),
-                             path ? path : "/", numstr);
+      if (date)
+         cmd = a_Dpip_build_cmd("cmd=%s cookie=%s host=%s path=%s date=%s",
+                                "set_cookie", cookie_string,
+                                URL_HOST_(set_url), path ? path : "/", date);
+      else
+         cmd = a_Dpip_build_cmd("cmd=%s cookie=%s host=%s path=%s",
+                                "set_cookie", cookie_string,
+                                URL_HOST_(set_url), path ? path : "/");
 
       _MSG("Cookies.c: a_Cookies_set \n\t \"%s\" \n",cmd );
       /* This call is commented because it doesn't guarantee the order
@@ -177,26 +179,34 @@ void a_Cookies_set(Dlist *cookie_strings, const DilloUrl *set_url)
 /*
  * Return a string containing cookie data for an HTTP query.
  */
-char *a_Cookies_get_query(const DilloUrl *request_url)
+char *a_Cookies_get_query(const DilloUrl *query_url, const DilloUrl *requester)
 {
-   char *cmd, *dpip_tag, *query, numstr[16];
+   char *cmd, *dpip_tag, *query;
    const char *path;
    CookieControlAction action;
 
    if (disabled)
       return dStrdup("");
 
-   action = Cookies_control_check(request_url);
+   action = Cookies_control_check(query_url);
    if (action == COOKIE_DENY) {
-      _MSG("Cookies: denied GET for %s\n", URL_HOST_(request_url));
+      _MSG("Cookies: denied GET for %s\n", URL_HOST_(query_url));
       return dStrdup("");
    }
-   path = URL_PATH_(request_url);
 
-   snprintf(numstr, 16, "%d", URL_PORT(request_url));
-   cmd = a_Dpip_build_cmd("cmd=%s scheme=%s host=%s path=%s port=%s",
-                          "get_cookie", URL_SCHEME(request_url),
-                         URL_HOST(request_url), path ? path : "/", numstr);
+   if (requester == NULL) {
+      /* request made by user */
+   } else if (!a_Url_same_organization(query_url, requester)) {
+      MSG("Cookies: not sent for request by '%s' for '%s'\n",
+          URL_HOST(requester), URL_HOST(query_url));
+      return dStrdup("");
+   }
+
+   path = URL_PATH_(query_url);
+
+   cmd = a_Dpip_build_cmd("cmd=%s scheme=%s host=%s path=%s",
+                          "get_cookie", URL_SCHEME(query_url),
+                         URL_HOST(query_url), path ? path : "/");
 
    /* Get the answer from cookies.dpi */
    _MSG("cookies.c: a_Dpi_send_blocking_cmd cmd = {%s}\n", cmd);
@@ -204,15 +214,11 @@ char *a_Cookies_get_query(const DilloUrl *request_url)
    _MSG("cookies.c: after a_Dpi_send_blocking_cmd resp={%s}\n", dpip_tag);
    dFree(cmd);
 
-   query = dStrdup("Cookie2: $Version=\"1\"\r\n");
-
    if (dpip_tag != NULL) {
-      char *cookie = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cookie");
-      char *old_query = query;
-      query = dStrconcat(old_query, cookie, NULL);
-      dFree(old_query);
+      query = a_Dpip_get_attr(dpip_tag, "cookie");
       dFree(dpip_tag);
-      dFree(cookie);
+   } else {
+      query = dStrdup("");
    }
    return query;
 }
@@ -237,7 +243,6 @@ static int Cookie_control_init(void)
    char line[LINE_MAXLEN];
    char domain[LINE_MAXLEN];
    char rule[LINE_MAXLEN];
-   int i, j;
    bool_t enabled = FALSE;
 
    /* Get a file pointer */
@@ -262,21 +267,20 @@ static int Cookie_control_init(void)
       dStrstrip(line);
 
       if (line[0] != '\0' && line[0] != '#') {
-         i = 0;
-         j = 0;
+         int i = 0, j = 0;
 
          /* Get the domain */
-         while (line[i] != '\0' && !isspace(line[i]))
+         while (line[i] != '\0' && !dIsspace(line[i]))
             domain[j++] = line[i++];
          domain[j] = '\0';
 
          /* Skip past whitespaces */
-         while (isspace(line[i]))
+         while (dIsspace(line[i]))
             i++;
 
          /* Get the rule */
          j = 0;
-         while (line[i] != '\0' && !isspace(line[i]))
+         while (line[i] != '\0' && !dIsspace(line[i]))
             rule[j++] = line[i++];
          rule[j] = '\0';
 
@@ -298,8 +302,17 @@ static int Cookie_control_init(void)
             default_action = cc.action;
             dFree(cc.domain);
          } else {
+            int i;
+            uint_t len = strlen(cc.domain);
+
+            /* Insert into list such that longest rules come first. */
             a_List_add(ccontrol, num_ccontrol, num_ccontrol_max);
-            ccontrol[num_ccontrol++] = cc;
+            for (i = num_ccontrol++;
+                 i > 0 && (len > strlen(ccontrol[i-1].domain));
+                 i--) {
+               ccontrol[i] = ccontrol[i-1];
+            }
+            ccontrol[i] = cc;
          }
 
          if (cc.action != COOKIE_DENY)
@@ -313,7 +326,9 @@ static int Cookie_control_init(void)
 }
 
 /*
- * Check the rules for an appropriate action for this domain
+ * Check the rules for an appropriate action for this domain.
+ * The rules are ordered by domain length, with longest first, so the
+ * first match is the most specific.
  */
 static CookieControlAction Cookies_control_check_domain(const char *domain)
 {
