@@ -22,9 +22,9 @@
 #include "misc.h"
 #include "msg.h"
 #include "prefs.h"
-#include "nav.h"
 #include "uicmd.hh"
 
+using namespace lout;
 using namespace dw;
 using namespace dw::core;
 using namespace dw::core::style;
@@ -248,6 +248,10 @@ static void Html_add_input(DilloHtml *html, DilloHtmlInputType type,
       int ni = html->inputs_outside_form->size();
       html->inputs_outside_form->increase();
       html->inputs_outside_form->set(ni, input);
+
+      if (html->bw->NumPendingStyleSheets > 0) {
+         input->setEnabled(false);
+      }
    }
 }
 
@@ -256,18 +260,20 @@ static void Html_add_input(DilloHtml *html, DilloHtmlInputType type,
  */
 static DilloHtmlInput *Html_get_radio_input(DilloHtml *html, const char *name)
 {
-   lout::misc::SimpleVector<DilloHtmlInput*>* inputs;
+   if (name) {
+      lout::misc::SimpleVector<DilloHtmlInput*>* inputs;
 
-   if (html->InFlags & IN_FORM)
-      inputs = html->getCurrentForm()->inputs;
-   else
-      inputs = html->inputs_outside_form;
+      if (html->InFlags & IN_FORM)
+         inputs = html->getCurrentForm()->inputs;
+      else
+         inputs = html->inputs_outside_form;
 
-   for (int idx = 0; idx < inputs->size(); idx++) {
-      DilloHtmlInput *input = inputs->get(idx);
-      if (input->type == DILLO_HTML_INPUT_RADIO &&
-          input->name && !dStrcasecmp(input->name, name))
-         return input;
+      for (int idx = 0; idx < inputs->size(); idx++) {
+         DilloHtmlInput *input = inputs->get(idx);
+         if (input->type == DILLO_HTML_INPUT_RADIO &&
+             input->name && !dStrcasecmp(input->name, name))
+            return input;
+      }
    }
    return NULL;
 }
@@ -548,6 +554,12 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
 //          gtk_entry_set_max_length(GTK_ENTRY(widget),
 //                                   strtol(attrbuf, NULL, 10));
       }
+      if (prefs.show_tooltip &&
+          (attrbuf = a_Html_get_attr(html, tag, tagsize, "title"))) {
+
+         html->styleEngine->setNonCssHint (PROPERTY_X_TOOLTIP, CSS_TYPE_STRING,
+                                           attrbuf);
+      }
       HT2TB(html)->addWidget (embed, html->styleEngine->backgroundStyle());
    }
    dFree(type);
@@ -599,7 +611,6 @@ void Html_tag_open_isindex(DilloHtml *html, const char *tag, int tagsize)
 
 /*
  * The textarea tag
- * (TODO: It doesn't support wrapping).
  */
 void Html_tag_open_textarea(DilloHtml *html, const char *tag, int tagsize)
 {
@@ -840,7 +851,10 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
       Embed *embed;
       char *name, *value;
 
-      page = new Textblock (prefs.limit_text_width);
+      /* We used to have Textblock (prefs.limit_text_width) here,
+       * but it caused 100% CPU usage.
+       */
+      page = new Textblock (false);
       page->setStyle (html->styleEngine->backgroundStyle ());
 
       ResourceFactory *factory = HT2LT(html)->getResourceFactory();
@@ -853,8 +867,6 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
       HT2TB(html)->addParbreak (5, html->styleEngine->wordStyle ());
 
       S_TOP(html)->textblock = html->dw = page;
-      /* right button press for menus for button contents */
-      html->connectSignals(page);
 
       value = a_Html_get_attr_wdef(html, tag, tagsize, "value", NULL);
       name = a_Html_get_attr_wdef(html, tag, tagsize, "name", NULL);
@@ -919,7 +931,7 @@ DilloHtmlForm::~DilloHtmlForm ()
 
 void DilloHtmlForm::eventHandler(Resource *resource, EventButton *event)
 {
-   MSG("DilloHtmlForm::eventHandler\n");
+   _MSG("DilloHtmlForm::eventHandler\n");
    if (event && (event->button == 3)) {
       a_UIcmd_form_popup(html->bw, html->page_url, this, showing_hiddens);
    } else {
@@ -946,10 +958,10 @@ void DilloHtmlForm::submit(DilloHtmlInput *active_input, EventButton *event)
             if (event->state == SHIFT_MASK) focus = !focus;
             a_UIcmd_open_url_nt(html->bw, url, focus);
          } else {
-            a_Nav_push_nw(html->bw, url);
+            a_UIcmd_open_url_nw(html->bw, url);
          }
       } else {
-         a_Nav_push(html->bw, url);
+         a_UIcmd_open_url(html->bw, url);
       }
       a_Url_free(url);
    }
@@ -1912,8 +1924,6 @@ static Embed *Html_input_image(DilloHtml *html, const char *tag, int tagsize)
          HT2TB(html)->addWidget (button, html->styleEngine->style ());
 //       gtk_widget_set_sensitive(widget, FALSE); /* Until end of FORM! */
 
-         /* a right button press brings up the image menu */
-         html->connectSignals((Widget*)Image->dw);
       } else {
          a_Url_free(url);
       }
@@ -1933,7 +1943,6 @@ static void Html_option_finish(DilloHtml *html)
        input->type == DILLO_HTML_INPUT_SEL_LIST) {
       DilloHtmlOption *option =
          input->select->getCurrentOption ();
-      option->content =
-         a_Html_parse_entities(html, html->Stash->str, html->Stash->len);
+      option->content = dStrndup(html->Stash->str, html->Stash->len);
    }
 }
