@@ -1,7 +1,6 @@
 /*
  * File: misc.c
  *
- * Copyright (C) 2000 Jrgen Viksell <vsksga@hotmail.com>
  * Copyright (C) 2000-2007 Jorge Arellano Cid <jcid@dillo.org>,
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,11 +13,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <assert.h>
 
 #include "utf8.hh"
 #include "msg.h"
 #include "misc.h"
-#include "utf8.hh"
 
 /*
  * Escape characters as %XX sequences.
@@ -51,38 +50,50 @@ char *a_Misc_escape_chars(const char *str, const char *esc_set)
 /*
  * Takes a string and converts any tabs to spaces.
  */
-char *a_Misc_expand_tabs(const char *str, int len)
+int
+a_Misc_expand_tabs(char **start, char *end, char *buf, int buflen)
 {
-   int i = 0, j, pos = 0, old_pos, char_len;
+   int j, pos = 0, written = 0, old_pos, char_len;
    uint_t code;
-   char *val;
+   static const int combining_char_space = 32;
 
-   if (memchr(str, '\t', len) == NULL) {
-      val = dStrndup(str, len);
-   } else {
-      Dstr *New = dStr_new("");
+   while (*start < end && written < buflen - TAB_SIZE - combining_char_space) {
+      code = a_Utf8_decode(*start, end, &char_len);
 
-      while (i < len) {
-         code = a_Utf8_decode(&str[i], str + len, &char_len);
-
-         if (code == '\t') {
-            /* Fill with whitespaces until the next tab. */
-            old_pos = pos;
-            pos += TAB_SIZE - (pos % TAB_SIZE);
-            for (j = old_pos; j < pos; j++)
-               dStr_append_c(New, ' ');
-         } else {
-            dStr_append_l(New, &str[i], char_len);
-            pos++;
-         }
-
-         i += char_len;
+      if (code == '\t') {
+         /* Fill with whitespaces until the next tab. */
+         old_pos = pos;
+         pos += TAB_SIZE - (pos % TAB_SIZE);
+         for (j = old_pos; j < pos; j++)
+            buf[written++] = ' ';
+      } else {
+         assert(char_len <= 4);
+         for (j = 0; j < char_len; j++)
+            buf[written++] = (*start)[j];
+         pos++;
       }
 
-      val = New->str;
-      dStr_free(New, FALSE);
+      *start += char_len;
    }
-   return val;
+
+   /* If following chars are combining chars (e.g. accents) add them to the
+    * buffer. We have reserved combining_char_space bytes for this.
+    * If there should be more combining chars, we split nevertheless.
+    */
+   while (*start < end && written < buflen - 4) {
+      code = a_Utf8_decode(*start, end, &char_len);
+
+      if (! a_Utf8_combining_char(code))
+         break;
+
+      assert(char_len <= 4);
+      for (j = 0; j < char_len; j++)
+         buf[written++] = (*start)[j];
+
+      *start += char_len;
+   }
+
+   return written;
 }
 
 /* TODO: could use dStr ADT! */
@@ -93,6 +104,7 @@ typedef struct ContentType_ {
 
 static const ContentType_t MimeTypes[] = {
    { "application/octet-stream", 24 },
+   { "application/xhtml+xml", 21 },
    { "text/html", 9 },
    { "text/plain", 10 },
    { "image/gif", 9 },
@@ -128,7 +140,7 @@ int a_Misc_get_content_type_from_data(void *Data, size_t Size, const char **PT)
    DetectedContentType Type = DT_OCTET_STREAM; /* default to binary */
 
    /* HTML try */
-   for (i = 0; i < Size && isspace(p[i]); ++i);
+   for (i = 0; i < Size && dIsspace(p[i]); ++i);
    if ((Size - i >= 5  && !dStrncasecmp(p+i, "<html", 5)) ||
        (Size - i >= 5  && !dStrncasecmp(p+i, "<head", 5)) ||
        (Size - i >= 6  && !dStrncasecmp(p+i, "<title", 6)) ||
@@ -166,7 +178,7 @@ int a_Misc_get_content_type_from_data(void *Data, size_t Size, const char **PT)
       Size = MIN (Size, 256);
       for (i = 0; i < Size; i++) {
          int ch = (uchar_t) p[i];
-         if (ch < 32 && !isspace(ch))
+         if (ch < 32 && !dIsspace(ch))
             ++bin_chars;
          if (ch > 126)
             ++non_ascci;
@@ -192,12 +204,13 @@ int a_Misc_get_content_type_from_data(void *Data, size_t Size, const char **PT)
 
 /*
  * Parse Content-Type string, e.g., "text/html; charset=utf-8".
+ * Content-Type is defined in RFC 2045 section 5.1.
  */
-void a_Misc_parse_content_type(const char *str, char **major, char **minor,
+void a_Misc_parse_content_type(const char *type, char **major, char **minor,
                                char **charset)
 {
-   const char *s;
-   bool_t is_text;
+   static const char tspecials_space[] = "()<>@,;:\\\"/[]?= ";
+   const char *str, *s;
 
    if (major)
       *major = NULL;
@@ -205,22 +218,31 @@ void a_Misc_parse_content_type(const char *str, char **major, char **minor,
       *minor = NULL;
    if (charset)
       *charset = NULL;
-   if (!str)
+   if (!(str = type))
       return;
 
-   for (s = str; isalnum(*s) || (*s == '-'); s++);
+   for (s = str; *s && !iscntrl((uchar_t)*s) && !strchr(tspecials_space, *s);
+        s++) ;
    if (major)
       *major = dStrndup(str, s - str);
-   is_text = (s - str == 4) && !dStrncasecmp(str, "text", 4);
 
    if (*s == '/') {
-      for (str = ++s; isalnum(*s) || (*s == '-'); s++);
+      for (str = ++s;
+           *s && !iscntrl((uchar_t)*s) && !strchr(tspecials_space, *s); s++) ;
       if (minor)
          *minor = dStrndup(str, s - str);
    }
-
-   if (is_text && charset && *s) {
-      /* charset parameter is defined for text media type (RFC 2046) */
+   if (charset && *s &&
+       (dStrncasecmp(type, "text/", 5) == 0 ||
+        dStrncasecmp(type, "application/xhtml+xml", 21) == 0)) {
+      /* "charset" parameter defined for text media type in RFC 2046,
+       * application/xhtml+xml in RFC 3236.
+       *
+       * Note that RFC 3023 lists some main xml media types and provides
+       * the convention of using the "+xml" minor type suffix for other
+       * xml types, so it would be reasonable to check for that suffix if
+       * we have need to care about various xml types someday.
+       */
       const char terminators[] = " ;\t";
       const char key[] = "charset";
 
@@ -320,7 +342,12 @@ int a_Misc_content_type_check(const char *EntryType, const char *DetectedType)
               dStrncasecmp(DetectedType, "application/", 12)) {
       /* Not an application sent as text */
       st = 0;
+   } else if (dStrncasecmp(EntryType, "application/xhtml+xml", 21) &&
+              dStrncasecmp(DetectedType, "text/html", 9)) {
+      /* XML version of HTML */
+      st = 0;
    }
+   _MSG("Type check: %s\n", st == 0 ? "MATCH" : "MISMATCH");
 
    return st;
 }
@@ -355,6 +382,40 @@ int a_Misc_parse_geometry(char *str, int *x, int *y, int *w, int *h)
    return ret;
 }
 
+/*
+ * Parse dillorc's search_url string ("[<label> ]<url>")
+ * Return value: -1 on error, 0 on success (and label and urlstr pointers)
+ */
+int a_Misc_parse_search_url(char *source, char **label, char **urlstr)
+{
+   static char buf[32];
+   char *p, *q;
+   int ret = -1;
+
+   if ((p = strrchr(source, ' '))) {
+      /* label and url pair */
+      strncpy(buf,source,MIN(p-source,31));
+      buf[MIN(p-source,31)] = 0;
+      source = p+1;
+      if ((p = strchr(source, '/')) && p[1] && (q = strchr(p+2,'/'))) {
+         *urlstr = source;
+         ret = 0;
+      }
+   } else {
+      /* url only, make a custom label */
+      if ((p = strchr(source, '/')) && p[1] && (q = strchr(p+2,'/'))) {
+         strncpy(buf,p+2,MIN(q-p-2,31));
+         buf[MIN(q-p-2,31)] = 0;
+         *urlstr = source;
+         ret = 0;
+      }
+   }
+   *label = buf;
+   if (ret == -1)
+      MSG("Invalid search_url: \"%s\"\n", source);
+   return ret;
+}
+
 /*
  * Encodes string using base64 encoding.
  * Return value: new string or NULL if input string is empty.
